@@ -57,6 +57,8 @@
 
 #define LS_LOGRAW(...) log4cxx::Logger::s_lograw( __VA_ARGS__);
 
+#define LS_LOG_F(...) log4cxx::Logger::s_log_forced( __VA_ARGS__);
+
 #define LS_DBG_IO( ... ) \
     do { \
         if ( log4cxx::Level::isEnabled( log4cxx::Level::DBG_IODATA ) ) \
@@ -377,6 +379,12 @@ public:
 #endif
         ;
 
+    static void s_log_forced(int level, LogSession *pLogSession, const char *format, ...)
+#if __GNUC__
+        __attribute__((format(printf, 3, 4)))
+#endif
+        ;
+
     static void s_logbuf(log4cxx::Logger *l, const char *buf, size_t size);
 
     static void s_vlograw(log4cxx::Logger *l, const char *format, va_list va);
