@@ -25,7 +25,7 @@
 #include <util/autostr.h>
 #include <quic/pbset.h>
 #include <quic/quicshm.h>   /* For _NOT_USE_SHM_ definition */
-#include <lsr/reuseport.h>
+#include <socket/reuseport.h>
 
 class QuicEngine;
 class VHostMap;
@@ -56,14 +56,14 @@ public:
         , m_id(-1)
         , m_pPacketsIn(NULL)
     {}
-        
+
     ~UdpListener();
 
     int setAddr(const char *pAddr);
     int setAddr(GSockAddr *);
-    
+
     int start();
-    
+
     ssize_t sendPacket(struct iovec *, size_t iovlen, const struct sockaddr *,
                                         const struct sockaddr *, int ecn);
     ssize_t send(struct iovec *, size_t iovlen, const struct sockaddr *src,
@@ -73,9 +73,9 @@ public:
     virtual int handleEvents(short int event);
 
     int onRead();
-    
+
     static void onMyEvent(int signo, void *param);
-    
+
     QuicEngine *getEngine() { return m_pEngine; }
 
     virtual const char *buildLogId();
@@ -84,23 +84,23 @@ public:
     static void initCidListenerHt();
     static void deleteCidListenerEntry(const lsquic_cid_t *);
     CidInfo *updateCidListenerMap(const lsquic_cid_t *, ShmCidPidInfo *pShmInfo);
-    
+
     void setId(int id)      {   m_id = id;        }
     int  getId() const      {   return m_id;      }
     HttpListener *getTcpPeer() const { return m_pTcpPeer; }
     unsigned short getPort() const { return m_addr.getPort(); }
 
     struct ssl_ctx_st *getSslContext() const;
-    
+
     int setSockOptions(int fd);
-    
-    
+
+
     void initShmCidMapping();
     int beginServe(QuicEngine *pEngine);
     int bind2(int flag);
     int bind();
-    
-    
+
+
     int getFdCount(int* maxfd);
     int passFds(int target_fd, const char *addr_str);
     void addReusePortSocket(int fd, const char *addr_str);
