@@ -317,9 +317,9 @@ void *LsShmHash::getObsData(LsShmHElem *pElem) const
 }
 
 
-LsShmOffset_t LsShmHash::alloc2(LsShmSize_t size, int &remapped)
+LsShmOffset_t LsShmHash::alloc2(LsShmSize_t size)
 {
-    LsShmOffset_t ret = m_pPool->alloc2(size, remapped);
+    LsShmOffset_t ret = m_pPool->alloc2(size);
     if (ret != 0)
         getHTable()->x_stat.m_iHashInUse += LsShmPool::size2roundSize(size);
     return ret;
@@ -457,9 +457,7 @@ LsShmOffset_t LsShmHash::allocHTable(LsShmPool * pPool, int init_size,
                                      int iMode, int iFlags,
                                      LsShmOffset_t lockOffset)
 {
-    int remapped;
-    // NOTE: system is not up yet... ignore remap here
-    LsShmOffset_t offset = pPool->alloc2(sizeof(LsShmHTable), remapped);
+    LsShmOffset_t offset = pPool->alloc2(sizeof(LsShmHTable));
     if (offset == 0)
     {
         return 0;
@@ -468,7 +466,7 @@ LsShmOffset_t LsShmHash::allocHTable(LsShmPool * pPool, int init_size,
     init_size = roundUp(init_size);
     int szTable = sz2TableSz(init_size);
     int szBitMap = sz2BitMapSz(init_size);
-    LsShmOffset_t iBase = pPool->alloc2(szTable + szBitMap, remapped);
+    LsShmOffset_t iBase = pPool->alloc2(szTable + szBitMap);
 
     if (iBase == 0)
     {
@@ -698,8 +696,8 @@ int LsShmHash::rehash()
     LsShmSize_t newSize;
     LsShmOffset_t newIdxOff;
     LsShmOffset_t newBitOff;
-    LsShmHIterOff *pIdxOld;
-    LsShmHIterOff *pIdxNew;
+    LsShmHIterOff *pOldTbl;
+    LsShmHIterOff *pNewTbl;
     LsShmHIterOff *opIdx;
     LsShmHIterOff *npIdx;
     iterator iter;
@@ -717,61 +715,64 @@ int LsShmHash::rehash()
               );
 #endif
     LsShmHTable *pTable = getHTable();
-    pIdxOld = (LsShmHIterOff *)m_pPool->offset2ptr(pTable->x_iHIdx);
+    pOldTbl = (LsShmHIterOff *)m_pPool->offset2ptr(pTable->x_iHIdx);
     if (pTable->x_iHIdx != pTable->x_iHIdxNew)          // rehash in progress
     {
         newSize = pTable->x_iCapacityNew;
         newIdxOff = pTable->x_iHIdxNew;
-        pIdxNew = (LsShmHIterOff *)m_pPool->offset2ptr(newIdxOff);
+        pNewTbl = (LsShmHIterOff *)m_pPool->offset2ptr(newIdxOff);
         if ((iterOff.m_iOffset = pTable->x_iWorkIterOff) != 0)    // iter in progress
         {
             iter = offset2iterator(iterOff);
-            npIdx = pIdxNew + getIndex(iter->x_hkey, newSize);
+            npIdx = pNewTbl + getIndex(iter->x_hkey, newSize);
             if (npIdx->m_iOffset != iterOff.m_iOffset)            // not there yet
             {
-                opIdx = pIdxOld + getIndex(iter->x_hkey, oldSize);
+                opIdx = pOldTbl + getIndex(iter->x_hkey, oldSize);
                 if (opIdx->m_iOffset == iterOff.m_iOffset)
                     opIdx->m_iOffset = iter->x_iNext.m_iOffset;   // remove from old
                 iter->x_iNext.m_iOffset = npIdx->m_iOffset;
                 npIdx->m_iOffset = iterOff.m_iOffset;
             }
-
         }
     }
     else
     {
-        int remapped;
         newSize = s_primeList[findRange(oldSize) + growFactor()];
         szTable = sz2TableSz(newSize);
         szBitMap = sz2BitMapSz(newSize);
-        if ((newBitOff = alloc2(szTable + szBitMap, remapped)) == 0)
+        if ((newBitOff = alloc2(szTable + szBitMap)) == 0)
             return LS_FAIL;
         uint8_t *ptr = (uint8_t *)offset2ptr(newBitOff);
         ::memset(ptr, 0, szTable + szBitMap);
         newIdxOff = newBitOff + szBitMap;
-        pIdxNew = (LsShmHIterOff *)(ptr + szBitMap);
+        pNewTbl = (LsShmHIterOff *)(ptr + szBitMap);
         pTable = getHTable();
         pTable->x_iBitMap = newBitOff;
         pTable->x_iBitMapSz = szBitMap;
         pTable->x_iCapacityNew = newSize;
         pTable->x_iHIdxNew = newIdxOff;
-        iterOff = begin();
     }
 
-    pIdxOld = (LsShmHIterOff *)m_pPool->offset2ptr(pTable->x_iHIdx);
+    iterOff = begin();
+    pOldTbl = (LsShmHIterOff *)m_pPool->offset2ptr(pTable->x_iHIdx);
     while(iterOff.m_iOffset != 0)
     {
-        uint32_t hashIndx;
+        uint32_t new_idx;
         iter = offset2iterator(iterOff);
+        opIdx = pOldTbl + getIndex(iter->x_hkey, oldSize);
         iterNextOff = next(iterOff);
-        hashIndx = getIndex(iter->x_hkey, newSize);
-        npIdx = pIdxNew + hashIndx;
-        setBitMapEnt(hashIndx);
+        new_idx = getIndex(iter->x_hkey, newSize);
+        npIdx = pNewTbl + new_idx;
+        setBitMapEnt(new_idx);
+
         pTable->x_iWorkIterOff = iterOff.m_iOffset;
-        (pIdxOld + getIndex(iter->x_hkey, oldSize))->m_iOffset = iter->x_iNext.m_iOffset;
+        assert(opIdx->m_iOffset == iterOff.m_iOffset);
+        opIdx->m_iOffset = iter->x_iNext.m_iOffset;
+        assert(npIdx->m_iOffset != iterOff.m_iOffset);
         iter->x_iNext.m_iOffset = npIdx->m_iOffset;
         npIdx->m_iOffset = iterOff.m_iOffset;
         iterOff = iterNextOff;
+
         if (++count > oldSize + oldSize / 2)
         {
             fprintf(stderr, "LsShmHash::rehash() is in a infinity loop, likely due to SHM corruption. remove corrupted file.");
@@ -931,6 +932,7 @@ void LsShmHash::remove(iteroffset iterOff, iterator iter)
     LsShmHElem *pElem;
     LsShmOffset_t next = iter->x_iNext.m_iOffset;     // in case of remap in tid list
 
+    validateIndexSlot(hashIndx);
 
     //NOTE:race condition, two process release the object at the same time.
     //     ShmHash was not properly locked.
@@ -956,6 +958,8 @@ void LsShmHash::remove(iteroffset iterOff, iterator iter)
     {
         m_pTidMgr->eraseIterCb(iter);
         pIdx = getHidx(hashIndx);
+
+        validateIndexSlot(hashIndx);
     }
     if (offset == iterOff.m_iOffset)
     {
@@ -975,12 +979,15 @@ void LsShmHash::remove(iteroffset iterOff, iterator iter)
             // next offset...
             offset = pElem->x_iNext.m_iOffset;
         }
+        validateIndexSlot(hashIndx);
+
     }
 
     decrTableSize();
     if (m_iFlags & LSSHM_FLAG_LRU)
     {
         removeFromLru(iter);
+        validateIndexSlot(hashIndx);
         if (getLru()->n_current != getHTable()->x_iSize)
         {
             LsHashLruInfo info = *getLru();
@@ -1057,9 +1064,8 @@ LsShmHash::iteroffset LsShmHash::allocIter(int keyLen, int realValLen)
     int valLen = realValLen + m_dataExtraSpace;
     LsShmHElemLen_t elementSize = sizeof(LsShmHElem) + valueOff
                       + sizeof(ls_vardata_t) + round4(valLen);
-    int remapped;
     iteroffset offset;
-    offset.m_iOffset = alloc2(elementSize, remapped);
+    offset.m_iOffset = alloc2(elementSize);
     if (offset.m_iOffset == 0)
         return offset;
     LsShmHElem *pNew = (LsShmHElem *)m_pPool->offset2ptr(offset.m_iOffset);
@@ -1120,9 +1126,12 @@ void LsShmHash::insertAlloced(iteroffset iterOff, iterator iter)
 {
     uint32_t hashIndx = getIndex(iter->x_hkey, capacity());
     LsShmHIterOff *pIdx = getHidx(hashIndx);
+
+    validateIndexSlot(hashIndx);
     iter->x_iNext.m_iOffset = pIdx->m_iOffset;
     pIdx->m_iOffset = iterOff.m_iOffset;
     setBitMapEnt(hashIndx);
+    validateIndexSlot(hashIndx);
 
 #ifdef DEBUG_RUN
     SHM_NOTICE("LsShmHash::insert %6d %X size %d cap %d <%p> %d",
@@ -1138,6 +1147,7 @@ void LsShmHash::insertAlloced(iteroffset iterOff, iterator iter)
     if (m_iFlags & LSSHM_FLAG_LRU)
     {
         addToLru(iter, iterOff);
+        validateIndexSlot(hashIndx);
         if (getLru()->n_current != getHTable()->x_iSize)
         {
             LsHashLruInfo info = *getLru();
@@ -1163,6 +1173,8 @@ void LsShmHash::replace(iteroffset oldIterOff, iterator oldIter,
     LsShmSize_t size = oldIter->x_iLen;
     uint32_t hashIndx = getIndex(oldIter->x_hkey, capacity());
     LsShmHIterOff *pIdx = getHidx(hashIndx);
+    validateIndexSlot(hashIndx);
+
     LsShmOffset_t offset = pIdx->m_iOffset;
     LsShmHElem *pElem;
     LsShmOffset_t next = oldIter->x_iNext.m_iOffset;     // in case of remap in tid list
@@ -1192,6 +1204,7 @@ void LsShmHash::replace(iteroffset oldIterOff, iterator oldIter,
     {
         m_pTidMgr->eraseIterCb(oldIter);
         pIdx = getHidx(hashIndx);
+        validateIndexSlot(hashIndx);
     }
 
     if (offset == oldIterOff.m_iOffset)
@@ -1199,6 +1212,7 @@ void LsShmHash::replace(iteroffset oldIterOff, iterator oldIter,
         pIdx->m_iOffset = newIterOff.m_iOffset;
         newIter->x_iNext.m_iOffset = next;
         oldIter->x_iNext.m_iOffset = 0;
+        validateIndexSlot(hashIndx);
     }
     else
     {
@@ -1210,6 +1224,7 @@ void LsShmHash::replace(iteroffset oldIterOff, iterator oldIter,
                 pElem->x_iNext.m_iOffset = newIterOff.m_iOffset;
                 newIter->x_iNext.m_iOffset = next;
                 oldIter->x_iNext.m_iOffset = 0;
+                validateIndexSlot(hashIndx);
                 break;
             }
             // next offset...
@@ -1221,6 +1236,8 @@ void LsShmHash::replace(iteroffset oldIterOff, iterator oldIter,
     {
         removeFromLru(oldIter);
         addToLru(newIter, newIterOff);
+        validateIndexSlot(hashIndx);
+
         if (getLru()->n_current != getHTable()->x_iSize)
         {
             LsHashLruInfo info = *getLru();
@@ -1244,9 +1261,8 @@ LsShmHash::iteroffset LsShmHash::iterGrowValue(iteroffset iterOff,
     int valLen = pOld->getValLen();
     int valOff = pOld->x_iValOff;
     int newTotalSize = pOld->x_iLen + round4(size_to_grow);
-    int remapped;
     iteroffset offset;
-    offset.m_iOffset = alloc2(newTotalSize, remapped);
+    offset.m_iOffset = alloc2(newTotalSize);
     if (offset.m_iOffset == 0)
         return offset;
     LsShmHElem *pNew = offset2iterator(offset);
@@ -1260,26 +1276,31 @@ LsShmHash::iteroffset LsShmHash::iterGrowValue(iteroffset iterOff,
     pOld = (LsShmHElem *)m_pPool->offset2ptr(iterOff.m_iOffset);
         // pNew->x_iNext.m_iOffset = 0;
     pNew->x_hkey = pOld->x_hkey;
-
     setIterKey(pNew, pOld->getKey());
+    uint32_t hashIndx = getIndex(pNew->x_hkey, capacity());
+
     if (front)
         front = size_to_grow;
     ::memcpy(pNew->getVal() + front, pOld->getVal(), pOld->getValLen());
 
     eraseIteratorHelper(iterOff);
     if (m_iFlags & LSSHM_FLAG_LRU)
+    {
         addToLru(pNew, offset);
+        validateIndexSlot(hashIndx);
+    }
     if (m_pTidMgr != NULL && isTidMaster())
     {
         m_pTidMgr->insertIterCb(offset);
         pNew = offset2iterator(offset);
+        validateIndexSlot(hashIndx);
     }
 
-    uint32_t hashIndx = getIndex(pNew->x_hkey, capacity());
     LsShmHIterOff *pIdx = getHidx(hashIndx);
     pNew->x_iNext.m_iOffset = pIdx->m_iOffset;
     pIdx->m_iOffset = offset.m_iOffset;
     setBitMapEnt(hashIndx);
+    validateIndexSlot(hashIndx);
 
     incrTableSize();
     return offset;
@@ -1938,7 +1959,7 @@ LsShmHash::iteroffset LsShmHash::tid2IterOff(uint64_t tid) const
 }
 
 
-uint64_t LsShmHash::nextTidVal(uint64_t tidIn, void *&pSearchState,
+uint64_t LsShmHash::nextTidVal(uint64_t tidIn, LsShmTidTblBlk *&pSearchState,
         LsShmHash::iteroffset &outOffset, uint64_t &outDelTid) const
 {
     outOffset.m_iOffset = 0;
@@ -1959,6 +1980,26 @@ uint64_t LsShmHash::nextTidVal(uint64_t tidIn, void *&pSearchState,
 }
 
 
+int LsShmHash::getTidVal(uint64_t tid, LsShmTidTblBlk **pCurBlk,
+        iteroffset *outOffset, uint64_t *outDelTid) const
+{
+    outOffset->m_iOffset = 0;
+    *outDelTid = 0;
+    if (NULL == m_pTidMgr)
+        return -1;
+    assert(m_pPool->getShm()->isLocked(m_pShmLock));
+    uint64_t tid_val;
+    if (m_pTidMgr->getTidVal(tid, pCurBlk, &tid_val) != -1)
+    {
+        if (m_pTidMgr->isTidValIterOff(tid_val))
+            outOffset->m_iOffset = m_pTidMgr->tidVal2iterOff(tid_val);
+        else
+            *outDelTid = tid_val;
+    }
+    return -1;
+}
+
+
 void LsShmHash::assignTid(LsShmHash::iteroffset iterOff, uint64_t tid)
 {
     if (NULL == m_pTidMgr)
@@ -2120,6 +2161,40 @@ int LsShmHash::statIdx(iteroffset iterOff, for_each_fn2 fun, void *pUdata)
     return numInIdx;
 }
 
+#if DEBUG_SHM_HASH
+int LsShmHash::validateIndexSlot(uint32_t index)
+{
+    LsShmHIterOff offset = *getHidx(index);
+    LsShmHElem *pElem;
+    int count = 0;
+    LsShmOffset_t history[4096];
+
+    while (offset.m_iOffset != 0)
+    {
+        history[count++] = offset.m_iOffset;
+        if (count >= (int)(sizeof(history) / sizeof(LsShmOffset_t)))
+        {
+            //history overflow, should not happen.
+            abort();
+        }
+        else
+        {
+            for(int i = 0; i < count - 1; ++i)
+            {
+                //detect duplicate.
+                if (history[i] == offset.m_iOffset)
+                    abort();
+            }
+        }
+        pElem = (LsShmHElem *)m_pPool->offset2ptr(offset.m_iOffset);
+        //must fall into the same slot
+        if (pElem->x_hkey % capacity() != index)
+            abort();
+        offset.m_iOffset = pElem->x_iNext.m_iOffset;
+    }
+    return LS_OK;
+}
+#endif
 
 //
 //  @brief stat - populate the statistic of the hash table
